@@ -16,7 +16,7 @@ from spatialdata.transformations import Affine, Identity, MapAxis, Scale, Sequen
 from spatialdata.transformations._utils import _set_transformations
 
 import spatialdata_plot  # noqa: F401
-from tests.conftest import DPI, PlotTester, PlotTesterMeta
+from tests.conftest import DPI, PlotTester, PlotTesterMeta, _viridis_with_under_over
 
 RNG = np.random.default_rng(seed=42)
 sc.pl.set_rcParams_defaults()
@@ -456,6 +456,51 @@ class TestShapes(PlotTester, metaclass=PlotTesterMeta):
 
         sdata_blobs.pl.render_shapes("blobs_circles", color="instance_id").pl.show()
 
+    def test_plot_can_color_with_norm_no_clipping(self, sdata_blobs_shapes_annotated: SpatialData):
+        sdata_blobs_shapes_annotated.pl.render_shapes(
+            element="blobs_polygons", color="value", norm=Normalize(2, 4, clip=False), cmap=_viridis_with_under_over()
+        ).pl.show()
+
+    def test_plot_datashader_can_color_with_norm_and_clipping(self, sdata_blobs_shapes_annotated: SpatialData):
+        sdata_blobs_shapes_annotated.pl.render_shapes(
+            element="blobs_polygons",
+            color="value",
+            norm=Normalize(2, 4, clip=True),
+            cmap=_viridis_with_under_over(),
+            method="datashader",
+            datashader_reduction="max",
+        ).pl.show()
+
+    def test_plot_datashader_can_color_with_norm_no_clipping(self, sdata_blobs_shapes_annotated: SpatialData):
+        sdata_blobs_shapes_annotated.pl.render_shapes(
+            element="blobs_polygons",
+            color="value",
+            norm=Normalize(2, 4, clip=False),
+            cmap=_viridis_with_under_over(),
+            method="datashader",
+            datashader_reduction="max",
+        ).pl.show()
+
+    def test_plot_datashader_norm_vmin_eq_vmax_without_clip(self, sdata_blobs_shapes_annotated: SpatialData):
+        sdata_blobs_shapes_annotated.pl.render_shapes(
+            element="blobs_polygons",
+            color="value",
+            norm=Normalize(3, 3, clip=False),
+            cmap=_viridis_with_under_over(),
+            method="datashader",
+            datashader_reduction="max",
+        ).pl.show()
+
+    def test_plot_datashader_norm_vmin_eq_vmax_with_clip(self, sdata_blobs_shapes_annotated: SpatialData):
+        sdata_blobs_shapes_annotated.pl.render_shapes(
+            element="blobs_polygons",
+            color="value",
+            norm=Normalize(3, 3, clip=True),
+            cmap=_viridis_with_under_over(),
+            method="datashader",
+            datashader_reduction="max",
+        ).pl.show()
+
     def test_plot_can_annotate_shapes_with_table_layer(self, sdata_blobs: SpatialData):
         nrows, ncols = 5, 3
         feature_matrix = RNG.random((nrows, ncols))
